unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, ComCtrls, StdCtrls, NTFileSecurity;

type
  TfrmMain = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    ListView1: TListView;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    DirFull: TCheckBox;
    GroupBox3: TGroupBox;
    FFull: TCheckBox;
    FRead: TCheckBox;
    FWrite: TCheckBox;
    FExecute: TCheckBox;
    FDelete: TCheckBox;
    btnSetSecurity: TButton;
    DirRead: TCheckBox;
    DirWrite: TCheckBox;
    DirExecute: TCheckBox;
    DirDelete: TCheckBox;
    GroupBox4: TGroupBox;
    Edit1: TEdit;
    Label1: TLabel;
    Button1: TButton;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    lblHost: TLabel;
    lblVolume: TLabel;
    lblSerial: TLabel;
    lblFileSystem: TLabel;
    FPermissions: TCheckBox;
    FOwner: TCheckBox;
    DirPermissions: TCheckBox;
    DirOwner: TCheckBox;
    lblError: TLabel;
    btnAddEntry: TButton;
    btnDelEntry: TButton;
    FS: TNTFileSecurity;
    procedure Button1Click(Sender: TObject);
    procedure ListView1Click(Sender: TObject);
    procedure btnSetSecurityClick(Sender: TObject);
    procedure btnDelEntryClick(Sender: TObject);
    procedure btnAddEntryClick(Sender: TObject);
    procedure FSGetSecurity(Sender: TObject; Error, Location: Cardinal);
    procedure FSSetSecurity(Sender: TObject; Error, Location: Cardinal);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

procedure TfrmMain.Button1Click(Sender: TObject);
begin
  FS.FileName := '';
  FS.FileName := Edit1.Text;
end;

procedure TfrmMain.ListView1Click(Sender: TObject);
begin
  if ListView1.SelCount < 1 then Exit;

  FFull.Checked := (faAll in FS.FileRights[ListView1.Selected.Index]);
  FRead.Checked := (faRead in FS.FileRights[ListView1.Selected.Index]);
  FWrite.Checked := (faWrite in FS.FileRights[ListView1.Selected.Index]);
  FExecute.Checked := (faExecute in FS.FileRights[ListView1.Selected.Index]);
  FDelete.Checked := (faDelete in FS.FileRights[ListView1.Selected.Index]);
  FPermissions.Checked := (faChangePermissions in FS.FileRights[ListView1.Selected.Index]);
  FOwner.Checked := (faTakeOwnership in FS.FileRights[ListView1.Selected.Index]);

  DirFull.Checked := (faAll in FS.DirectoryRights[ListView1.Selected.Index]);
  DirRead.Checked := (faRead in FS.DirectoryRights[ListView1.Selected.Index]);
  DirWrite.Checked := (faWrite in FS.DirectoryRights[ListView1.Selected.Index]);
  DirExecute.Checked := (faExecute in FS.DirectoryRights[ListView1.Selected.Index]);
  DirDelete.Checked := (faDelete in FS.DirectoryRights[ListView1.Selected.Index]);
  DirPermissions.Checked := (faChangePermissions in FS.DirectoryRights[ListView1.Selected.Index]);
  DirOwner.Checked := (faTakeOwnership in FS.DirectoryRights[ListView1.Selected.Index]);
end;

procedure TfrmMain.btnSetSecurityClick(Sender: TObject);
var
  x: integer;
begin
  // TNTFileSecurity.FileRights and TNTFileSecurity.DirectoryRights both return
  // sets and set unions must be enclosed in brackets...
  // Correct:   [faRead] + [faWrite] = [faRead, faWrite] {Read/Write Access}
  //
  // Incorrect:
  //
  //      faRead  +  faWrite  =  faExecute        {Execute Rights}
  //     {Ord = 1    Ord = 2     Ord = 3}
  //
  //     [faRead] +  faWrite  =  Exception: Incompatible types

  if ListView1.SelCount < 1 then Exit;

  x := ListView1.Selected.Index;

  if FFull.Checked then
    FS.FileRights[x] := [faAll]
  else begin
    // Clear the existing set...
    FS.FileRights[x] := [];

    if FRead.Checked then FS.FileRights[x] := FS.FileRights[x] + [faRead];
    if FWrite.Checked then FS.FileRights[x] := FS.FileRights[x] + [faWrite];
    if FExecute.Checked then FS.FileRights[x] := FS.FileRights[x] + [faExecute];
    if FDelete.Checked then FS.FileRights[x] := FS.FileRights[x] + [faDelete];
    if FPermissions.Checked then FS.FileRights[x] := FS.FileRights[x] + [faChangePermissions];
    if FOwner.Checked then FS.FileRights[x] := FS.FileRights[x] + [faTakeOwnership];
  end;

  if DirFull.Checked then
    FS.DirectoryRights[x] := [faAll]
  else begin
    // Clear the existing set...
    FS.DirectoryRights[x] := [];
    if DirRead.Checked then FS.DirectoryRights[x] := FS.DirectoryRights[x] + [faRead];
    if DirWrite.Checked then FS.DirectoryRights[x] := FS.DirectoryRights[x] + [faWrite];
    if DirExecute.Checked then FS.DirectoryRights[x] := FS.DirectoryRights[x] + [faExecute];
    if DirDelete.Checked then FS.DirectoryRights[x] := FS.DirectoryRights[x] + [faDelete];
    if DirPermissions.Checked then FS.DirectoryRights[x] := FS.DirectoryRights[x] + [faChangePermissions];
    if DirOwner.Checked then FS.DirectoryRights[x] := FS.DirectoryRights[x] + [faTakeOwnership];
  end;

  FS.SetSecurity;
end;

procedure TfrmMain.btnDelEntryClick(Sender: TObject);
begin
  if ListView1.SelCount < 1 then Exit;
  FS.RemoveEntry(ListView1.Selected.Index);
  // The entry is not actually removed from the file/directory
  // until you call SetSecurity.  A call to RefreshSecurity will
  // clear the pending deletion.
  if MessageDlg('Are you sure you wish to remove the account?',mtConfirmation,[mbYes,mbNo],0) = mrYes then
    FS.SetSecurity
  else
    FS.RefreshSecurity;
end;

procedure TfrmMain.btnAddEntryClick(Sender: TObject);
var
  x: integer;
  Account: string;
begin
  // If you don't specify the server name for AddEntry, the process follows
  // the standard security search order.
  //
  //   1. built-in and administratively defined local accounts
  //   2. the primary domain
  //   3. trusted domains
  //
  // If the specified account is not found, AddEntry will raise ERROR_NONE_MAPPED.
  // If the specified server is not found, AddEntry will raise RPC_S_SERVER_UNAVAILABLE.

  try
    Account := InputBox('Security','Enter then account name:','');
    if Account <> '' then begin
      x := FS.AddEntry(FS.LocalMachine,Account);
      // You must set some rights.  Blank entries do not get added to the
      // Access Control List.  The added group/user will not show up in
      // TNTFileSecurity.Entries until the call to SetSecurity.
      FS.DirectoryRights[x] := [faRead];
      FS.FileRights[x] := [faRead];
      FS.SetSecurity;
    end;
  except
    on E: Exception do begin
      lblError.Caption := E.Message;
    end;
  end;
end;

procedure TfrmMain.FSGetSecurity(Sender: TObject; Error, Location: Cardinal);
var
  NewItem: TListItem;
  x: integer;
  Rights: string;
begin
  if Error <> ERROR_SUCCESS then begin
    // Initialize the display.
    ListView1.Items.Clear;
    lblHost.Caption := FS.HostName;
    lblVolume.Caption := FS.VolumeName;
    lblSerial.Caption := IntToStr(FS.VolumeSerialNumber);
    lblFileSystem.Caption := FS.FileSystemName;
    lblError.Caption := FS.GetMessageString(Error,Location);
  end else begin
    // Load the display with the retrieved information.
    ListView1.Items.Clear;
    lblError.Caption := '';
    lblHost.Caption := FS.HostName;
    lblVolume.Caption := FS.VolumeName;
    lblSerial.Caption := IntToStr(FS.VolumeSerialNumber);
    lblFileSystem.Caption := FS.FileSystemName;
    for x := 0 to Pred(FS.EntryCount) do begin
      NewItem := ListView1.Items.Add;
      NewItem.Caption := FS.EntryName[x];
      case FS.EntryType[x] of
        seNone           : NewItem.SubItems.Add('No Type');
        seUser           : NewItem.SubItems.Add('User');
        seGroup          : NewItem.SubItems.Add('Group');
        seDomain         : NewItem.SubItems.Add('Domain');
        seAlias          : NewItem.SubItems.Add('Alias (Local Group)');
        seWellKnownGroup : NewItem.SubItems.Add('Well Known Group');
        seDeletedAcount  : NewItem.SubItems.Add('Deleted Acount');
        seInvalid        : NewItem.SubItems.Add('Invalid Type');
        seUnknown        : NewItem.SubItems.Add('Unknown Type');
      end;
      NewItem.SubItems.Add(FS.ServerName[x]);
      Rights := '(';
      if (faAll in FS.FileRights[x]) then Rights := Rights + 'All'
      else begin
        if FS.FileRights[x] = [] then
          Rights := Rights + 'No Access'
        else begin
          if (faRead in FS.FileRights[x]) then Rights := Rights + 'R';
          if (faWrite in FS.FileRights[x]) then Rights := Rights + 'W';
          if (faExecute in FS.FileRights[x]) then Rights := Rights + 'X';
          if (faDelete in FS.FileRights[x]) then Rights := Rights + 'D';
          if (faChangePermissions in FS.FileRights[x]) then Rights := Rights + 'P';
          if (faTakeOwnership in FS.FileRights[x]) then Rights := Rights + 'O';
        end;
      end;
      Rights := Rights + ')';
      NewItem.SubItems.Add(Rights);

      Rights := '(';
      if (faAll in FS.DirectoryRights[x]) then Rights := Rights + 'All'
      else begin
        if FS.DirectoryRights[x] = [] then
          Rights := Rights + 'No Access'
        else begin
          if (faRead in FS.DirectoryRights[x]) then Rights := Rights + 'R';
          if (faWrite in FS.DirectoryRights[x]) then Rights := Rights + 'W';
          if (faExecute in FS.DirectoryRights[x]) then Rights := Rights + 'X';
          if (faDelete in FS.DirectoryRights[x]) then Rights := Rights + 'D';
          if (faChangePermissions in FS.DirectoryRights[x]) then Rights := Rights + 'P';
          if (faTakeOwnership in FS.DirectoryRights[x]) then Rights := Rights + 'O';
        end;
      end;
      Rights := Rights + ')';
      NewItem.SubItems.Add(Rights);
    end;
    ListView1Click(Self);
  end;
end;

procedure TfrmMain.FSSetSecurity(Sender: TObject; Error, Location: Cardinal);
begin
  if Error <> ERROR_SUCCESS then begin
    lblError.Caption := FS.GetMessageString(Error,Location);
  end else begin
    // Reload security information to reflect changes.
    FS.RefreshSecurity;
  end;
end;

end.
